// MRF89XA radio with 18F47J13 SPI2 using PPS
// SF v2.2.3.4
Device = 18F47J13
Clock = 8           // current intosc.bas v2.1 limit

Include "intosc.bas"
Include "setdigitalio.bas"

Include "usart.bas"
Include "convert.bas"

//-------------------------------------
// **HARDWARE DEPENDANT SECTION**
//-------------------------------------
// SPI2 (NOTE: if you change these you MUST change the PPS setup in OPEN_SPI2)  
Include "pps.bas"
'#option SPI2_SCK = PORTD.6         // SPI clock
'#option SPI2_SDI = PORTD.5         // SPI data in
'#option SPI2_SDO = PORTD.4         // SPI data out
#option SPI2_SCK = PORTB.4         // SPI clock
#option SPI2_SDI = PORTB.5         // SPI data in
#option SPI2_SDO = PORTB.3         // SPI data out
Include "SPI2.bas"

// IO pin assignments
Public Dim
'    R_RESET     As PORTD.0,             // to MRF89XA RESET pin
'    R_CSDAT     As PORTD.1,             // to MRF89XA CSDAT_N pin (active low)
'    R_CSCON     As PORTD.2,             // to MRF89XA CSCON_N pin (active low)
    R_RESET     As PORTB.0,             // to MRF89XA RESET pin
    R_CSDAT     As PORTA.0,             // to MRF89XA CSDAT_N pin (active low)
    R_CSCON     As PORTB.1,             // to MRF89XA CSCON_N pin (active low)
    R_SCK       As SPI2_SCK.SPI2_SCK@,  // SPI2 SCK
    R_SDO       As SPI2_SDO.SPI2_SDO@,  // SPI2 SDO (to MRF89XA SDI)
    R_SDI       As SPI2_SDI.SPI2_SDI@   // SPI2 SDI (from MRF89XA SDO)

// MRF89XA register addresses
Public Const
    GCON_REG    = $00,      // GENERAL CONFIGURATION REGISTER (ADDRESS:0X00)(POR:0X28)
    DMOD_REG    = $01,      // DATA AND MODULATION CONFIGURATION REGISTER (ADDRESS:0X01)(POR:0X88)   
    FDEV_REG    = $02,      // FREQUENCY DEVIATION CONTROL REGISTER (ADDRESS:0X02)(POR:0X03)
    BRS_REG     = $03,      // BIT RATE SET REGISTER (ADDRESS:0x03) (POR:0x07)
    FLTH_REG    = $04,      // FLOOR THRESHOLD CONTROL REGISTER (ADDRESS:0x04) (POR:0x0C)
    FIFOC_REG   = $05,      // FIFO CONFIGURATION REGISTER (ADDRESS:0x05) (POR:0x0F)
    R1C_REG     = $06,      // R1 COUNTER SET REGISTER (ADDRESS:0x06) (POR:0x77)
    P1C_REG     = $07,      // P1 COUNTER SET REGISTER (ADDRESS:0x07) (POR:0x64)
    S1C_REG     = $08,      // S1 COUNTER SET REGISTER (ADDRESS:0x08) (POR:0x32)
    R2C_REG     = $09,      // R2 COUNTER SET REGISTER (ADDRESS:0x09) (POR:0x74)
    P2C_REG     = $0A,      // P2 COUNTER SET REGISTER (ADDRESS:0x0A) (POR:0x62)
    S2C_REG     = $0B,      // S2 COUNTER SET REGISTER (ADDRESS:0x0B) (POR:0x32)
    PAC_REG     = $0C       // POWER AMPLIFIER CONTROL REGISTER (ADDRESS:0x0C) (POR:0x38)
    // OTHER REGISTERS CAN BE ADDED FROM THE DATASHEET 
        
//
// MRF89XA SPI settings (from datasheet DS70000622E)
// SPI_MODE_0
//  SMP = 0     'Input data is sampled at the middle of data output time
//  CKE = 1     'Transmit occurs on transition from the active to Idle clock state
//  CKP = 0     'Idle state for clock is a low level
//
Public Sub OPEN_SPI2()
  // max MRF89XA SCK is 1MHz for DATA mode
  #if (_clock <= 16)
    Const SPI_DIV = spiOscDiv16
  #else    
    Const SPI_DIV = spiOscDiv64
  #endif
  
    {
    // assign PPS for SPI2 (assumes 47J13 and PORTD)
    pps.unlock()
    pps.assign_input(PPS_SDI2, PPS_IN_RP22)     // SDI2 <- J13 PORTD.5
    pps.assign_output(PPS_SDO2, PPS_OUT_RP21)   // SDO2 -> J13 PORTD.4
    pps.assign_output(PPS_SCK2, PPS_OUT_RP23)   // SCK2 -> J13 PORTD.6
    pps.assign_input(PPS_SCK2IN, PPS_IN_RP23)   // SCK2 <- J13 PORTD.6
    }
    // assign PPS for SPI2 (assumes 47J13 and PORTB)
    pps.unlock()
    pps.assign_input(PPS_SDI2, PPS_IN_RP8)     // SDI2 <- J13 PORTB.5
    pps.assign_output(PPS_SDO2, PPS_OUT_RP6)   // SDO2 -> J13 PORTB.3
    pps.assign_output(PPS_SCK2, PPS_OUT_RP7)   // SCK2 -> J13 PORTB.4
    pps.assign_input(PPS_SCK2IN, PPS_IN_RP7)   // SCK2 <- J13 PORTB.4
    
    // init spi
    SPI2.Open(SPI_MODE_0, SPI_DIV, spiSampleMiddle)
End Sub

//
// An external hardware or manual reset of the MRF89XA can be performed
// by asserting the TEST8 pin (pin 13) to high for 100 s and then releasing
// the pin. After releasing the pin, it takes more than 5 ms for the 
// transceiver to be ready for any operations. The reset pin is driven 
// with an open-drain output and therefore, is pulled high while the device 
// is in POR. The device does not accept commands during the Reset period
//
Public Sub RADIO_RESET()
    High(R_RESET)
    DelayUS(150)        // wait RESET high time (>100us)
    Input(R_RESET)
    DelayMS(10)         // wait for transceiver ready (>5ms)
End Sub

// init IO pins, SPI, and reset MRF89XA
Public Sub RADIO_INIT()
    // setup CS pins
    High(R_CSCON)
    High(R_CSDAT)
    Input(R_RESET)
    
    // init SPI
    OPEN_SPI2()
    
    // reset MRF89XA
    RADIO_RESET()
End Sub

// MRF89XA interface
Public Inline Sub ASSERT_CSCON()
    R_CSCON = 0     // active low
    DelayUS(1)      // >500ns
End Sub

Public Inline Sub DEASSERT_CSCON()
    R_CSCON = 1
    DelayUS(1)      // >500ns
End Sub
    
Public Inline Sub ASSERT_CSDAT()
    R_CSDAT = 0     // active low
    DelayUS(1)      // >500ns
End Sub

Public Inline Sub DEASSERT_CSDAT()
    R_CSDAT = 1
    DelayUS(1)      // >500ns
End Sub

// MRF89XA CONFIG register format
//  bit 7: START bit = 0
//  bit 6: R/W 0=write, 1=read 
//  bits 5-1: register address A4-A0
//  bit 0: STOP bit = 0

// write config register
Public Sub MRF89XA_WRITE_REG(reg As Byte, val As Byte)
    // format control byte for write (add STOP=0, R/W=0)
    Dim control As Byte = (reg << 1)    // adds STOP=0
    control.bits(6) = 0                 // R/W = 0
    
    ASSERT_CSCON()
    SPI2.Transfer(control)
    SPI2.Transfer(val)
    DEASSERT_CSCON()
End Sub

// read config register
Public Function MRF89XA_READ_REG(reg As Byte) As Byte
    // format control byte for read (add STOP=0, R/W=1)
    Dim control As Byte = (reg << 1)    // adds STOP=0
    control.bits(6) = 1                 // R/W = 1
    
    ASSERT_CSCON()
    SPI2.Transfer(control)
    result = SPI2.Transfer()
    DEASSERT_CSCON()
End Function

// write data FIFO
Public Sub MRF89XA_WRITE_FIFO(val As Byte)
    ASSERT_CSDAT()
    SPI2.Transfer(val)
    DEASSERT_CSDAT()
End Sub

// read data FIFO
Public Function MRF89XA_READ_FIFO() As Byte
    ASSERT_CSDAT()
    result = SPI2.Transfer()
    DEASSERT_CSDAT()
End Function



Dim b As Byte

'These are how I have been setting the clock and disabling peripherals. I would 
'definately take any input on any improvements here. Once I got things to work I 
'just ran with it.
{
OSCCON   = $7F                         ' Interanl clock to 8mhz
OSCCON2  = $54
REFOCON  = $00
OSCTUNE  = $00                           
ADCON0   = $00                         ' A/D disabled
ADCON1   = $00                         ' A/D disabled
ANCON0   = $FF
ANCON1   = $1F
PMCONH   = $00                         ' Disable parallel port
DSCONH   = $00
DSCONL   = $05
ODCON1   = $00
ODCON2   = $00
CCP4CON  = $00
CCP5CON  = $00
CCP6CON  = $00
CCP7CON  = $00
CCP8CON  = $00
CCP9CON  = $00
CCP10CON = $00
CTMUCONH = $00                         ' Disable the CTMU

PMDIS3  = $FF                          ' Shut down peripherals
PMDIS2  = $FF
PMDIS1  = $E0                          ' F8
PMDIS0  = $F1                          ' Only ser1 and SPI1/2 enabled
}


main:
SetAllDigital()
USART.SetBaudrate(br19200)
USART.Write("MRF89XA SPI2 TEST",10,13)

RADIO_INIT()

b = MRF89XA_READ_REG(GCON_REG)
USART.Write("GCON = ",BinToStr(b,8),"  ",HexToStr(b),10,13)

MRF89XA_WRITE_REG(GCON_REG, $68)
b = MRF89XA_READ_REG(GCON_REG)
USART.Write("GCON = ",BinToStr(b,8),"  ",HexToStr(b),10,13)

MRF89XA_WRITE_REG(GCON_REG, $28)
b = MRF89XA_READ_REG(GCON_REG)
USART.Write("GCON = ",BinToStr(b,8),"  ",HexToStr(b),10,13)

MRF89XA_WRITE_REG(GCON_REG, $88)
b = MRF89XA_READ_REG(GCON_REG)
USART.Write("GCON = ",BinToStr(b,8),"  ",HexToStr(b),10,13)

